<?php

namespace Utopia\Validator;

use Utopia\Validator;

class Nullable extends Validator
{
    protected $validator;

    public function __construct($validator)
    {
        $this->validator = $validator;
    }

    /**
     * Get Description
     *
     * Returns validator description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->validator->getDescription() . ' or null';
    }

    /**
     * Is array
     *
     * Function will return true if object is array.
     *
     * @return bool
     */
    public function isArray()
    {
        return false;
    }

    /**
     * Get Type
     *
     * Returns validator type.
     *
     * @return string
     */
    public function getType()
    {
        return $this->validator->getType();
    }

    /**
     * @return Validator
     */
    public function getValidator()
    {
        return $this->validator;
    }

    /**
     * Is valid
     *
     * Validation will pass when $value is text with valid length.
     *
     * @param  mixed  $value
     * @return bool
     */
    public function isValid($value)
    {
        if (\is_null($value)) {
            return true;
        }

        return $this->validator->isValid($value);
    }
}
